-- Maclink.applescript
-- Maclink

--  Created by Nige on Mon Mar 08 2004.
--  Copyright (c) 2003 __MyCompanyName__. All rights reserved.

global thePath
global folderDepth
property folderContents : {}
property dataSource : missing value
property selectedDataItem : missing value
property selectedItem : missing value
property rootItem : missing value
property locatedItem : missing value
property selectedItemIsFolder : missing value
property itemIsFolder : missing value

to addRoot()
	set rootItem to make new data item at end of data items of dataSource
	set contents of data cell "name" of rootItem to "GP32"
	set contents of data cell "path" of rootItem to "/"
	set contents of data cell "size" of rootItem to "..."
end addRoot

to checkConnection(clearCache)
	tell drawer "drawer" of window "mainWindow" to open drawer on bottom edge
	try
		set macLink to "./maclink.pl"
		set content of text field "status" of window "mainWindow" to "Contacting GP32..."
		tell progress indicator "busy" of window "mainWindow" to start
		tell window "mainWindow" to update
		set theOutput to do shell script macLink & " info"
		tell progress indicator "busy" of window "mainWindow" to stop
		
		if theOutput is "ERROR: NO CONNECTION" then
			display dialog "GP32 not detected - is it connected?" buttons {"OK"}
		else if theOutput is "ERROR: NO LINK" then
			display dialog "Failed to talk to GP32 - is link mode enabled?" buttons {"OK"}
		else
			set contents of text view "output" of scroll view "output" of window "mainWindow" to theOutput
			set contents of text view "drawerContent" of scroll view "drawContent" of window "mainWindow" to theOutput
			set content of text field "status" of window "mainWindow" to "Reading contents of SMC..."
			tell progress indicator "busy" of window "mainWindow" to start
			tell window "mainWindow" to update
			
			if clearCache then
				set theOutput to do shell script macLink & " clear" -- Clear cache...
			end if
			
			set theSmcContents to do shell script "./maclink.pl ls"
			set theItems to split(theSmcContents, return)
			set dataSource to data source of outline view "items" of scroll view "scroll" of window "mainWindow"
			set update views of dataSource to false
			set visible of progress indicator "progress" of window "mainWindow" to true
			set content of text field "status" of window "mainWindow" to "Processing SMC contents..."
			
			delete every data item of dataSource
			addRoot()
			
			OpenFolder("", rootItem, 1, theItems)
			set update views of dataSource to true
			tell progress indicator "busy" of window "mainWindow" to stop
			set visible of progress indicator "progress" of window "mainWindow" to false
		end if
		
	on error msg
		tell progress indicator "busy" of window "mainWindow" to stop
		display dialog msg
	end try
	
	set content of text field "status" of window "mainWindow" to ""
end checkConnection

on clicked theObject
	set theOutput to ""
	set macLink to "./maclink.pl"
	
	-- Buttons
	if theObject is button "exit" of window "mainWindow" then
		do shell script macLink & " end"
		quit
		
	else if theObject is button "delete" of window "mainWindow" then
		if selectedItem is not missing value then
			try
				if selectedItemIsFolder is true then
					display dialog "Are you sure you want to delete the folder:" & return & "  " & selectedItem & " and all its contents?"
				else
					display dialog "Are you sure you want to delete:" & return & "  " & selectedItem & "?"
				end if
				tell progress indicator "busy" of window "mainWindow" to start
				set theOutput to do shell script macLink & " rm " & selectedItem
				if theOutput is "ERROR: FAILED TO REMOVE" then
					display dialog "Failed to delete:" & return & "  " & selectedItem
				else
					delete selectedDataItem
					set selectedItem to missing value
					set contents of text field "path" of window "mainWindow" to ""
					set theView to outline view "items" of scroll view "scroll" of window "mainWindow"
					set selected rows of theView to {0}
				end if
				
			on error msg
				if msg is not "User canceled." and msg is not "NSInternalScriptError" then
					display dialog msg
				end if
			end try
			tell progress indicator "busy" of window "mainWindow" to stop
		end if
		
	else if theObject is button "format" of window "mainWindow" then
		try
			display dialog "Are you sure you want to format the SMC card?" & return & "WARNING: All data will be permanently lost!" default button "Cancel"
			
			set theOutput to do shell script macLink & " format"
			
		on error msg
			if msg is not "User canceled." then
				display dialog msg
			end if
		end try
		
	else if theObject is button "refresh" of window "mainWindow" then
		checkConnection(true)
		
	end if
	
end clicked

on awake from nib theObject
	set visible of progress indicator "progress" of window "mainWindow" to false
	tell theObject to register drag types {"file names"}
end awake from nib

to fileExists(theFolder, filename)
	set theResult to false
	repeat with theItem in data items of theFolder
		set theItemPath to contents of data cell "path" of theItem
		set theItemName to contents of data cell "name" of theItem
		set theSize to contents of data cell "size" of theItem
		if theSize is "..." then
			set theFilename to theItemPath
		else
			set theFilename to theItemPath & "/" & theItemName
		end if
		if theFilename is filename then
			set locatedItem to theItem
			set theResult to true
			exit repeat
		end if
	end repeat
	return theResult
end fileExists

on drop theObject drag info dragInfo
	set dropped to false
	
	set dataSource to data source of outline view "items" of scroll view "scroll" of window "mainWindow"
	
	set thePath to contents of text field "path" of window "mainWindow"
	if thePath is not "" then
		if "file names" is in types of pasteboard of dragInfo then
			set theFiles to {}
			set preferred type of pasteboard of dragInfo to "file names"
			set theFiles to contents of pasteboard of dragInfo
			
			set macLink to "./maclink.pl"
			try
				tell progress indicator "busy" of window "mainWindow" to start
				tell window "mainWindow" to update
				set theCount to 0
				repeat with theFile in theFiles
					log "theFIle:" & theFile
					set itemIsFolder to true
					tell application "Finder"
						if class of theFile is folder or class of theFile is disk then
							set itemIsFolder to true
						else
							set itemIsFolder to false
						end if
					end tell
					log "IIF:" & itemIsFolder
					
					set theParts to split(theFile, "/")
					set theSimpleName to text item (count of theParts) of theParts
					
					set theComponents to split(theSimpleName, ".")
					set theNameLength to length of text item (1) of theComponents
					if theSimpleName contains "." then
						set theExtLength to length of text item (2) of theComponents
					else
						set theExtLength to 1
					end if
					
					if theNameLength < 1 or theNameLength > 8 or theExtLength < 1 or theExtLength > 3 then
						display dialog "Failed to copy " & theSimpleName & return & "    Filenames may only have up to 8 characters for" & return & "    the name and up to 3 characters for the extension." buttons {"OK"}
					else
						set theSimpleName to uppercase theSimpleName
						--						set theSimpleName to toUppercase(theSimpleName)
						set theDestination to thePath & "/" & theSimpleName
						try
							set continueCopy to true
							set overwritingFile to false
							
							if contents of data cell "parent" of selectedDataItem is "" then
								set theParent to dataSource
							else
								set theParent to contents of data cell "parent" of selectedDataItem
							end if
							
							if fileExists(theParent, theDestination) then
								if contents of data cell "size" of locatedItem is "..." then
									display dialog "Can't overwrite folder: " & theSimpleName buttons {"OK"}
									set continueCopy to false
								else
									display dialog "Overwrite " & theDestination & " on GP32?"
									set overwritingFile to true
								end if
							end if
							
							if continueCopy then
								set content of text field "status" of window "mainWindow" to "Copying " & theSimpleName & "..."
								tell window "mainWindow" to update
								log "File:" & theFile
								tell application "Finder"
									if class of theFile is folder or class of theFile is disk then
										set itemIsFolder to true
									else
										set itemIsFolder to false
									end if
								end tell
								
								if itemIsFolder then
									set theOutput to do shell script macLink & " mkdir " & thePath & "/" & theSimpleName
								else
									set theOutput to do shell script macLink & " ul " & thePath & " '" & theFile & "'"
								end if
								
								if theOutput is "ERROR: FAILED TO COPY" then
									display dialog "Failed to copy " & theFile buttons {"OK"}
								else
									if not overwritingFile then
										set newItem to make new data item at end of data items of theParent
										set contents of data cell "name" of newItem to theSimpleName
										set contents of data cell "path" of newItem to thePath
										if itemIsFolder then
											set contents of data cell "size" of newItem to "..."
										else
											set contents of data cell "size" of newItem to "?"
										end if
									end if
								end if
							end if
							
						on error msg
							if msg is not "User canceled." then
								display dialog msg
							end if
						end try
						
						tell window "mainWindow" to update
					end if
				end repeat
				
			on error msg
				display dialog msg
			end try
			
			set content of text field "status" of window "mainWindow" to ""
			tell progress indicator "busy" of window "mainWindow" to stop
			
			set dropped to true
		end if
	end if
	
	return dropped
end drop

on OpenFolder(theName, dataSource, theIndex, theItems)
	if theName is not "" then
		set parentItem to make new data item at end of data items of dataSource
		set thePath to split(theName, " ")
		set theParentPath to text item (1) of thePath
		set theSimpleName to text item (2) of thePath
		set contents of data cell "name" of parentItem to theSimpleName
		if theParentPath is "/" then
			set theParentPath to ""
		end if
		set contents of data cell "path" of parentItem to (theParentPath & "/" & theSimpleName)
		set contents of data cell "size" of parentItem to "..."
		set contents of data cell "parent" of parentItem to parentItem
	else
		set parentItem to dataSource
	end if
	repeat until theIndex > (count of theItems)
		set theItem to text item (theIndex) of theItems
		if length of theItem > 1 then
			set theName to text 2 through (length of theItem) of theItem
		end if
		if theItem starts with "<" then
			return theIndex + 1
		else if theItem starts with ">" then
			set theStats to split(theName, ":")
			set theIndex to OpenFolder(text item (1) of theStats, parentItem, theIndex + 1, theItems)
		else
			set theParts to split(theName, ":")
			set childItem to make new data item at end of data items of parentItem
			set contents of data cell "name" of childItem to text item (1) of theParts
			set contents of data cell "path" of childItem to theParentPath & "/" & theSimpleName
			set contents of data cell "size" of childItem to text item (2) of theParts
			set contents of data cell "parent" of childItem to parentItem
			
			set theIndex to theIndex + 1
		end if
		set contents of progress indicator "progress" of window "mainWindow" to (theIndex / (count of theItems)) * 100
	end repeat
end OpenFolder

on launched theObject
	set dataSource to make new data source at end of data sources with properties {name:"items"}
	tell dataSource
		make new data column at end of data columns with properties {name:"name"}
		make new data column at end of data columns with properties {name:"size"}
		make new data column at end of data columns with properties {name:"path"}
		make new data column at end of data columns with properties {name:"parent"}
	end tell
	
	addRoot()
	
	set data source of outline view "items" of scroll view "scroll" of window "mainWindow" to dataSource
	checkConnection(false)
end launched

on should select item theObject outline item outlineItem
	set selectedDataItem to outlineItem
	set theName to contents of data cell "name" of selectedDataItem
	set thePath to contents of data cell "path" of selectedDataItem
	set contents of text field "path" of window "mainWindow" to thePath
	if thePath is not "/" then
		set selectedItem to (thePath & "/" & theName)
	else
		set selectedItem to theName
	end if
	if contents of data cell "size" of selectedDataItem is "..." then
		set selectedItemIsFolder to true
	else
		set selectedItemIsFolder to false
	end if
	return true
end should select item

to split(someText, delimiter)
	set AppleScript's text item delimiters to delimiter
	set someText to someText's text items
	set AppleScript's text item delimiters to {""} --> restore delimiters to default value
	return someText
end split

property newChars : {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "_"}
property oldChars : {"a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", " "}

--on toUppercase(theText)
--	set originalDelimiters to AppleScript's text item delimiters
--	repeat with i from 1 to 27
--		set AppleScript's text item delimiters to (item i of oldChars)
--		set theText to text items of theText
--		set AppleScript's text item delimiters to (item i of newChars)
--		set theText to (theText as text)
--	end repeat
--	set AppleScript's text item delimiters to originalDelimiters
--	return theText
--end toUppercase

on will display browser cell theObject in column inColumn browser cell browserCell row theRow
	(*Add your script here.*)
end will display browser cell

on number of browser rows theObject in column inColumn
	(*Add your script here.*)
end number of browser rows

on prepare drop theObject drag info dragInfo
	(*Add your script here.*)
end prepare drop

on conclude drop theObject drag info dragInfo
	(*Add your script here.*)
end conclude drop

on drag theObject drag info dragInfo
	(*Add your script here.*)
end drag

on double clicked theObject
	(*Add your script here.*)
end double clicked

