#! /usr/bin/perl

my $notConnectedPattern = qr/failed to get device iterator/im;
my $linkModeFailurePattern = qr/failed to read GP32 information from GP32/im;
my $failedToRemovePattern = qr/Error: failed to remove/im;
my $failedToCopyPattern = qr/Error: failed to copy file/im;

my $statusPattern = qr/Total (\d*) entries and (\d*) bytes/;

unlink "maclink.cache" if( $ARGV[ 0 ] eq "clear" );

$output = `./maclink info 2>&1` if( $ARGV[ 0 ] eq "info" );
$output = `./maclink end 2>&1` if( $ARGV[ 0 ] eq "end" );
$output = `./maclink rm $ARGV[ 1 ] 2>&1` if( $ARGV[ 0 ] eq "rm" );
$output = `./maclink ul $ARGV[ 1 ] \'$ARGV[ 2 ]\' 2>&1` if( $ARGV[ 0 ] eq "ul" );
$output = `./maclink mkdir $ARGV[ 1 ] 2>&1` if( $ARGV[ 0 ] eq "mkdir" );
$output = `./maclink format 2>&1` if( $ARGV[ 0 ] eq "format" );

if( $ARGV[ 0 ] eq "ls" ) 
{
	if( -e "maclink.cache" ) # Read cache...
	{
		open( FILE, "maclink.cache" );
		@output = <FILE>;
		close( FILE );
		$output = join( "", @output );
	}
	else
	{
		$output = &ReadFolders if ($ARGV[ 0 ] eq "ls" );
		$errorStatus = &CheckStatus( $output );
		if( $errorStatus eq "OK" )
		{
			open( FILE, ">maclink.cache" ); # Cache output of ls...
			print FILE $output;		
			close( FILE );
		}
	}
}

$errorStatus = &CheckStatus( $output );
print $errorStatus eq "OK"  ? $output : $errorStatus;

exit;

sub ReadFolder
{
	my $name = shift;

	my $rawOutput = `./maclink ls -l $name 2>&1`;

	$errorStatus = &CheckStatus( $rawOutput );
	return $errorStatus if( $errorStatus ne "OK" );
	
	my @lines = split( /\n/, $rawOutput );
	@lines = @lines[ 2 .. $#lines ]; # Strip off Mr. Spiv's Header

	my $contents = "";

	my $status = @lines[ 0 ];

	if( $status =~ /$statusPattern/ ) # Extract stats
	{
		my @parts = split( /\//, $name );
		my $path = join( "/", @parts[ 0 .. $#parts - 1 ] );
		my $folderName = @parts[ $#parts ];
		$path = "/" if( $path eq "" );
		$contents .= ">".$path." ".$folderName.":".$1.":".$2."\n"  if( $folderName ne "" );
	}
	else
	{
		return $rawOutput;
	}
	
	foreach my $item (@lines[ 1 .. $#lines ])
	{
		$item =~ s/^\s+//;
        $item =~ s/\s+$//;
		$item =~ s/\s+/ /g;
		my @fields = split( / /, $item );
		my $filename = @fields[ 5 ];
		my $size = @fields[ 0 ];
		if( $size > 1024*1024 ) { $size = int( $size / (1024*1024) )." Mb"; }
		elsif( $size > 1024 ) { $size = int( $size / 1024 )." Kb"; }
		$contents .= substr( $filename, -1 ) eq "/" ? &ReadFolder( $name.$filename ) : "+".$filename.":".$size."\n";
	}

	$contents .= "<\n";
	
	return $contents;
}
sub ReadFolders
{
	return &ReadFolder( "/");
}

sub CheckStatus
{
	my $data = shift;
	return "ERROR: FAILED TO COPY" if( $output =~ /$failedToCopyPattern/ );
	return "ERROR: FAILED TO REMOVE" if( $output =~ /$failedToRemovePattern/ );
	return "ERROR: NO LINK" if( $output =~ /$linkModeFailurePattern/ );
	return "ERROR: NO CONNECTION" if( $output =~ /$notConnectedPattern/ );	
	return "OK";
}